@@ -1,5 +1,5 @@
 /*
- * Copyright 2013-2016 Fabian Groffen
+ * Copyright 2013-2017 Fabian Groffen
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -94,8 +94,6 @@ server_queuereader(void *d)
 	const char *p;
 
 	*metric = NULL;
-	__sync_and_and_fetch(&(self->metrics), 0);
-	__sync_and_and_fetch(&(self->ticks), 0);
 
 #define FAIL_WAIT_TIME   6  /* 6 * 250ms = 1.5s */
 #define DISCONNECT_WAIT_TIME   12  /* 12 * 250ms = 3s */
@@ -224,6 +222,9 @@ server_queuereader(void *d)
 			if (__sync_bool_compare_and_swap(&(self->keep_running), 0, 0))
 				break;
 			usleep((200 + (rand() % 100)) * 1000);  /* 200ms - 300ms */
+			/* avoid overflowing */
+			if (__sync_add_and_fetch(&(self->failure), 0) > FAIL_WAIT_TIME)
+				__sync_sub_and_fetch(&(self->failure), 1);
 		}
 
 		/* at this point we've got work to do, if we're instructed to
@@ -429,7 +430,7 @@ server_queuereader(void *d)
 						__sync_fetch_and_add(&(self->failure), 1) == 0)
 					logerr("failed to write() to %s:%u: %s\n",
 							self->ip, self->port,
-							(slen < 0 ? strerror(errno) : "uncomplete write"));
+							(slen < 0 ? strerror(errno) : "incomplete write"));
 				close(self->fd);
 				self->fd = -1;
 				/* put back stuff we couldn't process */
@@ -710,6 +711,16 @@ server_swap_queue(server *l, server *r)
 	t = l->queue;
 	l->queue = r->queue;
 	r->queue = t;
+	
+	/* swap associated statistics as well */
+	l->metrics = r->metrics;
+	l->dropped = r->dropped;
+	l->stalls = r->stalls;
+	l->ticks = r->ticks;
+	l->prevmetrics = r->prevmetrics;
+	l->prevdropped = r->prevdropped;
+	l->prevstalls = r->prevstalls;
+	l->prevticks = r->prevticks;
 }
 
 /**
